@@ -1,7 +1,25 @@
 /* cdb.c: cdb command line tool
  *
- * This file is a part of tinycdb package by Michael Tokarev, mjt@corpit.ru.
- * Public domain.
+ * This file is a part of tinycdb package.
+ * Copyright (C) 2001-2023 Michael Tokarev <mjt+cdb@corpit.ru>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define _GNU_SOURCE	/* #define this even on Windows */
@@ -13,6 +31,7 @@
 # pragma warning(disable: 4996)
 #else
 # include <unistd.h>
+# include <signal.h>
 #endif
 
 #include <sys/types.h>
@@ -74,6 +93,8 @@ static inline unsigned char *ufgets(unsigned char *s, int size, FILE *f) {
 static unsigned char *buf;
 static unsigned blen;
 
+static char *cleanup_tmpname;
+
 static void
 #ifdef __GNUC__
 __attribute__((noreturn,format(printf,2,3)))
@@ -94,6 +115,8 @@ error(int errnum, const char *fmt, ...)
     fprintf(stderr, "%s: try `%s -h' for help\n", progname, progname);
   }
   fflush(stderr);
+  if (cleanup_tmpname)
+    unlink(cleanup_tmpname);
   exit(errnum ? 111 : 2);
 }
 
@@ -408,11 +431,11 @@ cmode(char *dbname, char *tmpname, int argc, char **argv, int flags, int perms)
   if (perms >= 0)
     umask(0);
   unlink(tmpname);
+  cleanup_tmpname = tmpname;
   fd = open(tmpname, O_RDWR|O_CREAT|O_EXCL|O_NOFOLLOW,
             perms >= 0 ? perms : 0666);
-  if (fd < 0)
+  if (fd < 0 || cdb_make_start(&cdb, fd) < 0)
     error(errno, "unable to create %s", tmpname);
-  cdb_make_start(&cdb, fd);
   allocbuf(4096);
   if (argc) {
     int i;
@@ -432,7 +455,8 @@ cmode(char *dbname, char *tmpname, int argc, char **argv, int flags, int perms)
     dofile(&cdb, stdin, "(stdin)", flags);
   if (cdb_make_finish(&cdb) != 0)
     error(errno, "cdb_make_finish");
-  close(fd);
+  if (close(fd) != 0)
+    error(errno, "close %s", tmpname);
   if (tmpname != dbname)
     if (rename(tmpname, dbname) != 0)
       error(errno, "rename %s->%s", tmpname, dbname);
@@ -511,6 +535,14 @@ int main(int argc, char **argv)
 
   argv += optind;
   argc -= optind;
+
+#ifdef SIGXFSZ
+  /* at least on some linux architectures, writing past file size limit makes
+   * the process to receive SIGXFSZ in addition to returning -1 EFBIG
+   * from write(). Ignore SIGXFSZ to be able to handle write errors */
+  signal(SIGXFSZ, SIG_IGN);
+#endif
+
   switch(mode) {
     case 'q':
       if (argc < 2) error(0, "no database or key to query specified");
