@@ -1,7 +1,25 @@
 /* cdb_make.c: basic cdb creation routines
  *
- * This file is a part of tinycdb package by Michael Tokarev, mjt@corpit.ru.
- * Public domain.
+ * This file is a part of tinycdb package.
+ * Copyright (C) 2001-2023 Michael Tokarev <mjt+cdb@corpit.ru>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <unistd.h>
@@ -9,15 +27,6 @@
 #include <string.h>
 #include "cdb_int.h"
 
-void
-cdb_pack(unsigned num, unsigned char buf[4])
-{
-  buf[0] = num & 255; num >>= 8;
-  buf[1] = num & 255; num >>= 8;
-  buf[2] = num & 255;
-  buf[3] = num >> 8;
-}
-
 int
 cdb_make_start(struct cdb_make *cdbmp, int fd)
 {
@@ -25,7 +34,7 @@ cdb_make_start(struct cdb_make *cdbmp, int fd)
   cdbmp->cdb_fd = fd;
   cdbmp->cdb_dpos = 2048;
   cdbmp->cdb_bpos = cdbmp->cdb_buf + 2048;
-  return 0;
+  return lseek(fd, 0, SEEK_SET);
 }
 
 int internal_function
@@ -152,11 +161,11 @@ cdb_make_finish_internal(struct cdb_make *cdbmp)
     cdb_pack(hpos[t], p + (t << 3));
     cdb_pack(hcnt[t], p + (t << 3) + 4);
   }
-  if (lseek(cdbmp->cdb_fd, 0, 0) != 0 ||
+  if (lseek(cdbmp->cdb_fd, 0, SEEK_SET) != 0 ||
       _cdb_make_fullwrite(cdbmp->cdb_fd, p, 2048) != 0)
     return -1;
 
-  return 0;
+  return fsync(cdbmp->cdb_fd);
 }
 
 static void
